/**
 * CS 6630 - project 2, part 4
 * Konstantin Shkurko <kshkurko@cs.utah.edu>
 * 09.16.12
 */

FloatTable data;
float dataMin, dataMax;
float plotX1, plotY1;
float plotX2, plotY2;
float labelX, labelY;

int rowCount;
int columnCount;
int currentColumn = 0;

int yearMin, yearMax;
int[] years;

int yearInterval = 10;
int volumeInterval = 10;
int volumeIntervalMinor = 5;

float[] tabLeft, tabRight;
float tabTop, tabBottom;
float tabPad = 10;
String[] tabNames;
int[] colors = { #5679C1, #56C156, #C17956 };

Integrator[] interpolators;

int selectedPlot = 1;

PFont titleFont;
PFont axisFont;
PFont axisTitleFont;

void setup() {
  size( 720, 405 );
  
  data = new FloatTable( "milk-tea-coffee.tsv" );
  rowCount = data.getRowCount();
  columnCount = data.getColumnCount()+1;
  
  years = int( data.getRowNames() );
  yearMin = years[0];
  yearMax = years[years.length-1];
  
  dataMin = 0;
  dataMax = ceil( data.getTableMax() / volumeInterval ) * volumeInterval;
  
  interpolators = new Integrator[ rowCount ];
  for( int row = 0; row < rowCount; row++ ) {
    float initialValue = data.getFloat( row, 0 );
    interpolators[row] = new Integrator( initialValue );
  }

  // Corners of the plotted time series
  plotX1 = 80;
  plotX2 = width - 30;
  labelX = 20;
  plotY1 = 60;
  plotY2 = height - 70;
  labelY = height - 20;

  titleFont = createFont( "Verdana", 30, true );
  axisFont = createFont( "Georgia", 20, true );
  axisTitleFont = createFont( "Verdana", 20, true );

  smooth();
}


void draw() {
  background( 224 );

  // Show the plot area as a white box
  fill( 255 );
  rectMode( CORNERS );
  noStroke();
  rect( plotX1, plotY1, plotX2, plotY2 );

  //drawTitle();
  drawTitleTabs();
  drawAxisLabels();

  drawVolumeLabels();
  drawYearLabels();
 
  for( int row = 0; row < rowCount; row++ ) {
    interpolators[row].update();
  }

  // Do this for all tabs other than summary
  if( currentColumn != columnCount-1 ) {
    
    stroke( #5679C1 );

    // draw data as points
    if( selectedPlot==1 || selectedPlot==2 ) {
      strokeWeight( 5 );
      drawDataPoints( currentColumn, true );
    }

    // draw data as a connected line
    if( selectedPlot==2 ) {
      noFill();
      strokeWeight( 0.5 );
      drawDataLine( currentColumn, true );
    }
  
    // draw data as a smoothed line
    if( selectedPlot==3 ) {
      noFill();
      strokeWeight( 2 );
      drawDataCurve( currentColumn, true );
    }
  
    // draw data as an area plot
    if( selectedPlot==4 ) {
      noStroke();
      fill( #5679C1 );
      drawDataArea( currentColumn, true );
      drawYearLabels();
    }
  
    // highlight the point
    drawDataHighlight( currentColumn, #C17956 );
  }
  
  // otherwise, we're doing the summary and need
  // to plot all columns
  // we'll allow all but area plots here (easier) :D
  else {
    
    for( int col = 0; col < columnCount-1; col++ ) {

      // select the main color
      stroke( colors[col%3] );
      
      // draw data as points
      if( selectedPlot==1 || selectedPlot==2 ) {
        strokeWeight( 5 );
        drawDataPoints( col, false );
      }

      // draw data as a connected line
      if( selectedPlot==2 ) {
        noFill();
        strokeWeight( 0.5 );
        drawDataLine( col, false );
      }
  
      // draw data as a smoothed line
      if( selectedPlot==3 || selectedPlot==4 ) {
        noFill();
        strokeWeight( 2 );
        drawDataCurve( col, false );
      }
    
      // highlight the point
      drawDataHighlight( col, colors[col%3] );
    }
    
    // draw legend
    textFont( axisTitleFont );
    textSize( 17 );
    float maxWidth = 0;
    for( int col = 0; col < columnCount-1; col++ ) {
      maxWidth = max( maxWidth, textWidth(tabNames[col]) );
    }
    
    rectMode( CORNER );
    noStroke();
    fill( 255, 200 );
    float leftPt = plotX2 - maxWidth - 10;
    rect( leftPt-30, plotY1+1, maxWidth+30, 20*(columnCount-1)+10 );
    
    strokeWeight( 2 );
    fill( 0 );
    textAlign( LEFT );
    for( int col = 0; col < columnCount-1; col++ ) {
      text( tabNames[col], leftPt, plotY1 + 20*col + 21 );
      stroke( colors[col%3] );
      float yLoc = plotY1 + 20*col + 15;
      line( leftPt-25, yLoc, leftPt-10, yLoc );
    }
  }
}


void drawTitle() {
  fill( 0 );
  textFont( titleFont );
  textSize( 20 );
  //textAlign( CENTER );
  textAlign( LEFT );
  String title = data.getColumnName( currentColumn );
  //text( title, (plotX1+plotX2)/2, plotY1 - 10 );
  text( title, plotX1, plotY1 - 10 );
}


void drawTitleTabs() {
  rectMode( CORNERS );
  noStroke();
  textFont( titleFont );
  textSize( 20 );
  textAlign( LEFT );
  
  float runningX = plotX1;
  tabTop = plotY1 - textAscent() - 15;
  tabBottom = plotY1;
  
  // On first use of this method, allocate space for an array
  // to store the values for the left and right edges of the tabs
  if( tabLeft == null ) {
    tabLeft = new float[ columnCount ];
    tabRight = new float[ columnCount ];
    tabNames = new String[ columnCount ];
    
    // precalculate all of the info for tabs
    for( int col = 0; col < columnCount-1; col++ ) {
      String title  = data.getColumnName( col );
      tabNames[col] = title;
      tabLeft[col]  = runningX;
      float titleWidth = textWidth( title );
      tabRight[col] = tabLeft[col] + tabPad + titleWidth + tabPad;
      runningX      = tabRight[col];
    }
    
    // final tab name -> summary
    tabNames[ columnCount-1 ] = "Summary";
    tabLeft[ columnCount-1 ]  = runningX;
    float titleWidth          = textWidth( tabNames[ columnCount-1 ] );
    tabRight[ columnCount-1 ] = tabLeft[ columnCount-1 ] + tabPad + titleWidth + tabPad;
  }

  runningX = plotX1;
  for( int col = 0; col < columnCount; col++ ) {
    String title = tabNames[col];

    // If the current tab, set its background white; otherwise use pale gray
    fill( col == currentColumn ? 255 : 224 );
    rect( tabLeft[col], tabTop, tabRight[col], tabBottom );

    // If the current tab,use black for the text; otherwise use dark gray
    fill( col == currentColumn ? 0 : 64 );
    text( title, runningX + tabPad, plotY1 - 10 );
    runningX = tabRight[col];
  }
}


void drawAxisLabels() {
  fill ( 0 );
  textFont( axisTitleFont );
  textSize( 17 );
  
  pushMatrix();
  textAlign( CENTER, CENTER );
  translate( labelX, (plotY1+plotY2)/2 );
  rotate( radians(-90));
  text( "Gallons consumed per capita", 0, 0 );
  popMatrix();

  textAlign( CENTER );
  text( "Year", (plotX1+plotX2)/2, labelY );
}


void drawYearLabels() {
  fill( 0 );
  textFont( axisFont );
  textSize( 12 );
  textAlign( CENTER, TOP );
  
  // Use thin, fray lines to draw the grid
  strokeWeight( 1 );
  for( int row = 0; row < rowCount; row++ ) {
    if( years[row] % yearInterval == 0 ) {
      float x = map( years[row], yearMin, yearMax, plotX1, plotX2 );
      text( years[row], x, plotY2 + 10 );

      // main line (but not left most)
      if( years[row] != yearMin ) {
        stroke( 224 );
        line( x, plotY1, x, plotY2 );
      }
      
      // tick mark
      stroke( 128 );
      line( x, plotY2 - 2, x, plotY2 + 4 );
    }
  }
}


void drawVolumeLabels() {
  fill( 0 );
  textFont( axisFont );
  textSize( 12 );
  strokeWeight( 1 );
  
  for( float v = dataMin; v <= dataMax; v += volumeIntervalMinor ) {
    if( v % volumeIntervalMinor == 0 ) {       // If a tick mark
      float y = map( v, dataMin, dataMax, plotY2, plotY1 );
      if( v % volumeInterval == 0 ) {          // If a major tick mark
        if( v == dataMin ) {
          textAlign( RIGHT );                  // Align by the bottom
        }
        else if( v == dataMax ) {
          textAlign( RIGHT, TOP );             // Align by the top
        }
        else {
          textAlign( RIGHT, CENTER );          // Center vertically
        }
        text( floor(v), plotX1 - 10, y );

        // main line (but not lowest one)
        if( v != dataMin ) {
          stroke( 224 );
          line( plotX1, y, plotX2, y );
        }
        
        // tick mark
        stroke( 128 );
        line( plotX1 - 4, y, plotX1 + 2, y );
      }
      else {
        // Commented out - too sitracting visually
        //line( plotX1 - 2, y, plotX1, y );  // Draw minor tick 
      }
    }
  }
}


void drawDataPoints( int col, boolean useInterpolator ) {
  for( int row = 0; row < rowCount; row++ ) {
    if( data.isValid( row, col) ) {
      float value = ( useInterpolator ? interpolators[row]._value : data.getFloat( row, col ) );
      float x = map( years[row], yearMin, yearMax, plotX1, plotX2 );
      float y = map( value, dataMin, dataMax, plotY2, plotY1 );
      point( x, y );
    }
  }
}


void drawDataLine( int col, boolean useInterpolator ) {
  beginShape();
  int rowCount = data.getRowCount();
  for( int row = 0; row < rowCount; row++ ) {
    if( data.isValid( row, col ) ) {
      float value = ( useInterpolator ? interpolators[row]._value : data.getFloat( row, col ) );
      float x = map( years[row], yearMin, yearMax, plotX1, plotX2 );
      float y = map( value, dataMin, dataMax, plotY2, plotY1 );
      vertex( x, y );
    }
  }
  endShape();
}


void drawDataCurve( int col, boolean useInterpolator ) {
  beginShape();
  int rowCount = data.getRowCount();
  for( int row = 0; row < rowCount; row++ ) {
    if( data.isValid( row, col ) ) {
      float value = ( useInterpolator ? interpolators[row]._value : data.getFloat( row, col ) );
      float x = map( years[row], yearMin, yearMax, plotX1, plotX2 );
      float y = map( value, dataMin, dataMax, plotY2, plotY1 );
      
      curveVertex( x, y );
      // Double the curve point for the start and stop
      if( (row == 0) || (row == rowCount-1) ) {
        curveVertex( x, y );
      }
    }
  }
  endShape();
}


void drawDataArea( int col, boolean curved ) {
  beginShape();
  for( int row = 0; row < rowCount; row++ ) {
    if( data.isValid( row, col ) ) {
      float value = interpolators[row]._value;
      float x = map( years[row], yearMin, yearMax, plotX1, plotX2 );
      float y = map( value, dataMin, dataMax, plotY2, plotY1 );
      
      if( curved==false ) {
        vertex( x, y );
      }
      else {
        curveVertex( x, y );
        // Double the curve point for the start and stop
        if( (row == 0) || (row == rowCount-1) ) {
          curveVertex( x, y );
        }
      }
    }
  }
  // Draw the lower-right and lower-left corners
  vertex( plotX2, plotY2 );
  vertex( plotX1, plotY2 );
  endShape( CLOSE );
}


void drawDataHighlight( int col, int strokeColor ) {
  for( int row = 0; row < rowCount; row++ ) {
    if( data.isValid( row, col ) ) {
      float value = data.getFloat( row, col );
      float x = map( years[row], yearMin, yearMax, plotX1, plotX2 );
      float y = map( value, dataMin, dataMax, plotY2, plotY1 );
      if( dist( mouseX, mouseY, x, y ) < 3 ) {
        
        // background rectangle
        textFont( axisFont, 11 );
        String label = nf( value, 0, 2 ) + " (" + years[row] + ")";
        float tw = textWidth( label );
        fill( 255, 200 );
        noStroke();
        rectMode( CORNER );
        rect( x - tw/2-2, y-22, tw+4, 17 );

        // point
        stroke( strokeColor );
        strokeWeight( 10 );
        point( x, y );
        
        // text
        fill( 0 );
        textAlign( CENTER );
        text( label, x, y-10 );
      }
    }
  }
}


void setCurrent( int col ) {
  currentColumn = col;
  
  // this only matters if we aren't going to the summary tab
  if( col != columnCount-1 ) {
    for( int row = 0; row < rowCount; row++ ) {
      interpolators[row].target( data.getFloat( row, col ) );
    }
  }
}


void keyPressed() {
  if( key == '[' ) {
    setCurrent( currentColumn > 0 ? currentColumn-1 : columnCount-1 );
  }
  else if( key == ']' ) {
    setCurrent( currentColumn < columnCount-1 ? currentColumn+1 : 0 );
  }
  
  // plot types
  else if( key == '1' ) {
    selectedPlot = 1;
  }
  else if( key == '2' ) {
    selectedPlot = 2;
  }
  else if( key == '3' ) {
    selectedPlot = 3;
  }
  else if( key == '4' ) {
    selectedPlot = 4;
  }
}


void mousePressed() {
  if( mouseY > tabTop && mouseY < tabBottom ) {
    for( int col = 0; col < columnCount; col++ ) {
      if( mouseX > tabLeft[col] && mouseX < tabRight[col] ) {
        setCurrent( col );
      }
    }
  }
}
